#!/usr/bin/env python
# -*- coding: utf-8 -*-

import functools

from flask import Blueprint, flash, g, redirect, render_template, session, url_for
from werkzeug.security import check_password_hash

from pmml_ui.auth.db import UserDB
from pmml_ui.auth.forms import LoginForm

blueprint = Blueprint("auth", __name__, url_prefix="/auth")


@blueprint.before_app_request
def load_logged_in_user():
    user_id = session.get("user_id")

    if user_id is None:
        g.user = None
    else:
        g.user = UserDB().get_user(username=user_id)


@blueprint.route("/login", methods=("GET", "POST"))
def login():
    login_form = LoginForm()
    if login_form.validate_on_submit():
        username = login_form.username.data
        password = login_form.password.data
        error = None
        user = UserDB().get_user(username=username)

        if user is None:
            error = "Incorrect username."
        elif not check_password_hash(user.password_hash, password):
            error = "Incorrect password."

        if error is None:
            session.clear()
            session["user_id"] = user.name
            return redirect(url_for("mailing_lists.index"))

        flash(error)

    return render_template("auth/login.html", login_form=login_form)


@blueprint.route("/logout")
def logout():
    session.clear()
    return redirect(url_for("auth.login"))


def login_required(view):
    @functools.wraps(view)
    def wrapped_view(**kwargs):
        if g.user is None:
            return redirect(url_for("auth.login"))

        return view(**kwargs)

    return wrapped_view



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import urllib.parse


class MailingListMember:
    def __init__(self, address, name=None):
        self.name = name
        self.address = address
        if name is None:
            self.name = self.address.replace("@", "_at_")
            self.name = urllib.parse.quote_plus(self.name)

    def __repr__(self):
        return self.address


class MailingList:
    def __init__(self, address, members, name=None):
        self.name = name
        self.address = address
        self.members = members
        if name is None:
            self.name = self.address.split("@")[0]

    def get_member(self, address):
        for member in self.members:
            if member.address == address:
                return member

    def add_member(self, address):
        if self.get_member(address) is None:
            member = MailingListMember(address=address)
            self.members.append(member)

    def remove_member(self, address, address_is_url_encoded=True):
        decoded_address = address
        if address_is_url_encoded:
            decoded_address = urllib.parse.unquote_plus(address)
        member = self.get_member(address=decoded_address)
        self.members.remove(member)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from pathlib import Path

from werkzeug.security import generate_password_hash

from pmml_ui.auth.models import User
from pmml_ui.config import USER_FILE_PATH


class UserDB:
    def __init__(self, user_file_path=USER_FILE_PATH):
        self.file_path = Path(user_file_path)
        self.users = self.load()

    def load(self):
        if self.file_path.exists():
            with open(self.file_path) as users_file:
                users = json.load(users_file)
        else:
            users = {}
        return users

    def get_user(self, username):
        user = None
        if username in self.users:
            user = User(
                name=username, password_hash=self.users[username]["password_hash"]
            )
        return user

    def save(self):
        with open(self.file_path, "w") as users_file:
            json.dump(self.users, users_file)

    def add_user(self, username, password):
        password_hash = generate_password_hash(password)
        if username in self.users:
            raise KeyError("User already exists.")
        self.users[username] = {"password_hash": password_hash}
        self.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import json
import re

from kubernetes import client, config
from pmml_ui.config import K8S_CONNECTION_TYPE

CONNECTION_TYPE_LOCAL = "local"
CONNECTION_TYPE_INCLUSTER = "incluster"


class JsonSecret:
    def __init__(self, name, namespace, secret, data_path):
        self._data_path = data_path
        self._api_v1_secret = secret
        self._decoded_secret_data = base64.b64decode(secret.data[data_path])
        self._encoded_secret_data = secret.data[data_path]
        self.name = name
        self.namespace = namespace
        self.data = json.loads(self._decoded_secret_data)

    def update(self):
        self._decoded_secret_data = json.dumps(self.data)
        self._encoded_secret_data = base64.b64encode(
            self._decoded_secret_data.encode()
        ).decode("utf-8")
        self._api_v1_secret.data[self._data_path] = self._encoded_secret_data


class Pod:
    def __init__(self, name, namespace):
        self.name = name
        self.namespace = namespace


class Client:
    def __init__(self, connection_type=K8S_CONNECTION_TYPE):
        self.connection_type = connection_type
        self.api_v1 = self.setup(connection_type)

    @staticmethod
    def setup(connection_type):
        if connection_type == CONNECTION_TYPE_LOCAL:
            config.load_kube_config()
        elif connection_type == CONNECTION_TYPE_INCLUSTER:
            config.load_incluster_config()
        return client.CoreV1Api()

    def read_json_secret(self, name, namespace, data_path):
        k8s_secret = self.api_v1.read_namespaced_secret(name, namespace)
        return JsonSecret(name, namespace, k8s_secret, data_path)

    def replace_json_secret(self, secret):
        return self.api_v1.replace_namespaced_secret(
            secret.name, secret.namespace, secret._api_v1_secret
        )

    def delete_pod(self, name, namespace):
        self.api_v1.delete_namespaced_pod(name, namespace)

    def delete_matching_pods_in_namespace(self, namespace, pattern):
        pods_in_namespace = self.api_v1.list_namespaced_pod(namespace)
        compiled_pattern = re.compile(pattern)
        for pod in pods_in_namespace.items:
            if compiled_pattern.match(pod.metadata.name):
                self.delete_pod(pod.metadata.name, namespace)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pmml_ui.updater.k8s import Client, Pod
from pmml_ui.updater.models import PmmlConfiguration


class ConfigUpdater:
    def __init__(self, namespace, pod_name, secret_name, secret_filename):
        self.client = Client()
        secret = self.client.read_json_secret(secret_name, namespace, secret_filename)
        self.config = PmmlConfiguration(secret)
        self.pod = Pod(pod_name, namespace)

    def update(self, mailing_list):
        pod_pattern = f"^{self.pod.name}-.*$"
        self.config.update(mailing_list)
        self.client.replace_json_secret(self.config.json_secret)
        self.client.delete_matching_pods_in_namespace(
            pattern=pod_pattern, namespace=self.pod.namespace
        )

    def refresh(self):
        secret = self.client.read_json_secret(
            self.config.json_secret.name,
            self.config.json_secret.namespace,
            self.config.json_secret._data_path,
        )
        self.config = PmmlConfiguration(secret)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import Flask


def create_app():
    app = Flask(__name__)
    app.config.from_pyfile("config.py")

    from pmml_ui.auth.views import blueprint as auth_blueprint

    app.register_blueprint(auth_blueprint)

    from pmml_ui.mailing_lists.views import blueprint as mailing_lists_blueprint

    app.register_blueprint(mailing_lists_blueprint)
    app.add_url_rule("/", endpoint="mailing_lists.index")

    return app



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import getpass

from pmml_ui.auth.db import UserDB


def add_user():
    user_db = UserDB()
    username = input("User: ")
    password = getpass.getpass("Password: ")
    verify = getpass.getpass("Repeat password: ")
    if password != verify:
        print("Passwords do not match!")
    else:
        user_db.add_user(username, password)
        print("User added.")


if __name__ == "__main__":
    try:
        add_user()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pmml_ui.mailing_lists.models import MailingList, MailingListMember


class PmmlConfiguration:
    def __init__(self, json_secret):
        self.json_secret = json_secret
        self.mailing_lists = self.read_mailing_lists_from_json(self.json_secret)

    def read_mailing_lists_from_json(self, json_secret):
        mailing_lists = {}
        for mailing_list_address in json_secret.data:
            members = []
            for member_address in json_secret.data[mailing_list_address]["recipients"]:
                member = MailingListMember(address=member_address)
                members.append(member)
            mailing_list = MailingList(address=mailing_list_address, members=members)
            mailing_lists[mailing_list.name] = mailing_list
        return mailing_lists

    def update(self, mailing_list):
        recipients = [member.address for member in mailing_list.members]
        self.json_secret.data[mailing_list.address]["recipients"] = recipients
        self.json_secret.update()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask_wtf import FlaskForm
from wtforms import FieldList, Form, FormField, HiddenField, StringField
from wtforms.validators import DataRequired, Email

from pmml_ui.mailing_lists.models import MailingList, MailingListMember


class MailingListMemberForm(Form):
    name = HiddenField("name")
    address = StringField("address", validators=[DataRequired(), Email()])


class MailingListMemberFormCSRF(FlaskForm):
    name = HiddenField("name")
    address = StringField("address", validators=[DataRequired(), Email()])


class MailingListForm(FlaskForm):
    name = HiddenField("name")
    address = StringField("address", validators=[DataRequired(), Email()])
    members = FieldList(FormField(MailingListMemberForm))

    @property
    def mailing_list(self):
        member_objects = [
            MailingListMember(address=member.address.data) for member in self.members
        ]
        return MailingList(address=self.address.data, members=member_objects)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from pathlib import Path

SECRET_KEY = os.urandom(64)
DEFAULT_USER_FILE_PATH = f"{Path.home()}/.config/pmml-ui/users.json"
USER_FILE_PATH = Path(os.environ.get("PMML_USER_FILE_PATH", DEFAULT_USER_FILE_PATH))
K8S_CONNECTION_TYPE = os.environ.get("PMML_K8S_CONNECTION_TYPE", "local")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pmml_ui', 'pmml_ui.auth', 'pmml_ui.mailing_lists', 'pmml_ui.updater']

package_data = \
{'': ['*'],
 'pmml_ui': ['templates/*', 'templates/auth/*', 'templates/mailing_lists/*']}

install_requires = \
['Flask>=2.2.3,<3.0.0',
 'flask-wtf>=1.1.1,<2.0.0',
 'kubernetes>=26.1.0,<27.0.0',
 'werkzeug>=2.2.3,<3.0.0',
 'wtforms[email]>=3.0.1,<4.0.0']

entry_points = \
{'console_scripts': ['add-pmml-ui-user = pmml_ui.auth.add_user:add_user']}

setup_kwargs = {
    'name': 'pmml-ui',
    'version': '0.1.4',
    'description': 'Web ui for pmml',
    'long_description': '# pmmml-ui\n',
    'author': 'Michael Wilson',
    'author_email': 'mw@1wilson.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask_wtf import FlaskForm
from wtforms import PasswordField, StringField
from wtforms.validators import DataRequired


class LoginForm(FlaskForm):
    username = StringField("username", validators=[DataRequired()])
    password = PasswordField("password", validators=[DataRequired()])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import Blueprint, abort, g, redirect, render_template, session, url_for

from pmml_ui.auth.views import login_required
from pmml_ui.mailing_lists.forms import MailingListForm, MailingListMemberFormCSRF
from pmml_ui.updater.client import ConfigUpdater

blueprint = Blueprint("mailing_lists", __name__, url_prefix="/mailing_lists")


@blueprint.before_app_request
def setup_updater():
    if "user_id" in session:
        g.updater = ConfigUpdater("pmml", "pmml-deployment", "pmml-pmmlrc", ".pmmlrc")


@blueprint.route("/")
@login_required
def index():
    return render_template(
        "mailing_lists/list.html", mailing_lists=g.updater.config.mailing_lists.values()
    )


@blueprint.route("/<mailing_list_name>", methods=["GET", "POST"])
@login_required
def detail(mailing_list_name):
    if mailing_list_name not in g.updater.config.mailing_lists:
        abort(404, description="Resource not found")
    mailing_list = g.updater.config.mailing_lists[mailing_list_name]
    mailing_list_form = MailingListForm(obj=mailing_list)
    if mailing_list_form.validate_on_submit():
        g.updater.config.mailing_lists[
            mailing_list_name
        ] = mailing_list_form.mailing_list
        g.updater.update(mailing_list_form.mailing_list)
        return redirect(
            url_for("mailing_lists.detail", mailing_list_name=mailing_list_name)
        )
    return render_template(
        "mailing_lists/detail.html",
        mailing_list=mailing_list,
        mailing_list_form=mailing_list_form,
    )


@blueprint.route("/<mailing_list_name>/add", methods=["GET", "POST"])
@login_required
def add_member(mailing_list_name):
    if mailing_list_name not in g.updater.config.mailing_lists:
        abort(404, description="Resource not found")
    mailing_list = g.updater.config.mailing_lists[mailing_list_name]
    member_form = MailingListMemberFormCSRF()
    if member_form.validate_on_submit():
        mailing_list.add_member(address=member_form.address.data)
        g.updater.update(mailing_list)
        return redirect(
            url_for("mailing_lists.detail", mailing_list_name=mailing_list_name)
        )
    return render_template(
        "mailing_lists/add_member.html",
        member_form=member_form,
        mailing_list_name=mailing_list.name,
    )


@blueprint.route("/<mailing_list_name>/<member_address>/delete")
@login_required
def delete_member(mailing_list_name, member_address):
    if mailing_list_name not in g.updater.config.mailing_lists:
        abort(404, description="Resource not found")
    mailing_list = g.updater.config.mailing_lists[mailing_list_name]
    if mailing_list.get_member(member_address) is None:
        abort(404, description="Resource not found")
    mailing_list.remove_member(member_address)
    g.updater.update(mailing_list)
    return redirect(
        url_for("mailing_lists.detail", mailing_list_name=mailing_list_name)
    )
